package org.mercury_im.messenger.ui.chatlist;

import android.content.Context;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import org.mercury_im.messenger.R;
import org.mercury_im.messenger.persistence.pojo.Chat;
import org.mercury_im.messenger.ui.chat.ChatActivity;
import org.mercury_im.messenger.ui.util.AbstractRecyclerViewAdapter;
import org.mercury_im.messenger.util.ColorUtil;

import butterknife.BindView;
import butterknife.ButterKnife;

public class ChatListRecyclerViewAdapter
        extends AbstractRecyclerViewAdapter<Chat, ChatListRecyclerViewAdapter.ChatHolder> {

    public ChatListRecyclerViewAdapter() {
        super(new ChatMessageDiffCallback(true));
    }

    @NonNull
    @Override
    public ChatHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.list_item_chat, parent, false);
        return new ChatHolder(parent.getContext(), view);
    }

    @Override
    public void onBindViewHolder(@NonNull ChatHolder holder, int position) {
        Chat model = getModelAt(position);
        holder.nameView.setText(model.contactName != null ?
                model.contactName : model.jid.toString());
        holder.avatarView.setColorFilter(ColorUtil.consistentColor(model.jid.toString()));

        holder.itemView.setOnClickListener(view -> {

            Intent intent = new Intent(holder.context, ChatActivity.class);
            intent.putExtra(ChatActivity.EXTRA_JID, model.jid.toString());
            intent.putExtra(ChatActivity.EXTRA_ACCOUNT, model.accountId);

            holder.context.startActivity(intent);
        });
        // TODO: Better bindable model pls
    }

    public static class ChatHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.chat_name)
        TextView nameView;

        @BindView(R.id.chat_message)
        TextView chatMessage;

        @BindView(R.id.chat_time)
        TextView chatTime;

        @BindView(R.id.chat_avatar)
        ImageView avatarView;

        Context context;

        public ChatHolder(Context context, @NonNull View itemView) {
            super(itemView);
            this.context = context;
            ButterKnife.bind(this, itemView);
        }
    }

    private static class ChatMessageDiffCallback extends AbstractDiffCallback<Chat> {

        ChatMessageDiffCallback(boolean detectMoves) {
            super(detectMoves);
        }

        @Override
        public boolean areItemsTheSame(Chat oldItem, Chat newItem) {
            return oldItem.chatId == newItem.chatId;
        }

        @Override
        public boolean areContentsTheSame(Chat oldItem, Chat newItem) {
            return areItemsTheSame(oldItem, newItem);
        }
    }
}
