package org.mercury_im.messenger.persistence.repository;

import org.jxmpp.jid.EntityBareJid;
import org.mercury_im.messenger.persistence.model.MessageModel;

import java.util.List;

import io.reactivex.Maybe;
import io.reactivex.Observable;
import io.reactivex.Single;

public interface MessageRepository<E extends MessageModel> {

    E newMessageModel();

    Maybe<E> getMessage(long accountId, long chatId, long messageId);

    Single<Long> insertMessage(E message);

    Single<List<Long>> insertMessages(List<E> messages);

    Observable<List<E>> getAllMessages();

    Observable<List<E>> getAllMessagesOf(long accountId);

    Observable<List<E>> getAllMessagesOfChat(long accountId, EntityBareJid peer);

    Observable<List<E>> getAllMessagesFrom(long accountId, EntityBareJid contact);

    Observable<List<E>> findMessageByQuery(String query);

    Observable<List<E>> findMessageByQuery(long accountId, String query);

    Observable<List<E>> findMessageByQuery(long accountId, EntityBareJid peer, String query);

    Observable<E> getLastMessageFrom(long accountId, EntityBareJid peer);
}
